@@ -7,7 +7,6 @@ from sapiopylib.rest.pojo.datatype.FieldDefinition import FieldType, AbstractVel
 from sapiopylib.rest.pojo.eln.ElnExperiment import ElnExperiment
 from sapiopylib.rest.pojo.eln.ExperimentEntry import ExperimentEntry
 from sapiopylib.rest.pojo.eln.SapioELNEnums import ElnBaseDataType
-from sapiopylib.rest.pojo.eln.eln_headings import ElnExperimentTab
 from sapiopylib.rest.pojo.webhook.WebhookContext import SapioWebhookContext
 from sapiopylib.rest.utils.Protocols import ElnExperimentProtocol, ElnEntryStep
 from sapiopylib.rest.utils.recordmodel.PyRecordModel import PyRecordModel, AbstractRecordModel
@@ -224,9 +223,12 @@ class AliasUtil:
             # macros get translated to valid field values.
             fields: FieldMap = {f: record.fields.get(f) for f in record.fields}
         # PR-47457: Only include the record ID if the caller requests it, since including the record ID can break
-        # callbacks in certain circumstances if the record ID is negative.
+        # callbacks in certain circumstances.
+        # PR-47894: Also remove the RecordId key if it exists and the caller doesn't want it included.
         if include_record_id:
             fields["RecordId"] = AliasUtil.to_record_id(record)
+        elif "RecordId" in fields:
+            del fields["RecordId"]
         return fields
 
     @staticmethod
