@@ -1,19 +1,30 @@
 from sapiopylib.rest.User import SapioUser
-from sapiopylib.rest.pojo.webhook.WebhookContext import SapioWebhookContext
 
-from sapiopycommons.general.aliases import RecordIdentifier, AliasUtil, ExperimentIdentifier
+from sapiopycommons.general.aliases import RecordIdentifier, AliasUtil, ExperimentIdentifier, DataTypeIdentifier, \
+    UserIdentifier
 
 
 class ProcessTracking:
+    """
+    A class for calling the Foundations process tracking endpoints.
+
+    IMPORTANT NOTICE: This is only for experiment processes that make use of AssignedProcess records. For custom
+    processes that use ProcessQueueItem records, see the QueueItemHandler class.
+    """
+    # PR-47941: Update the assign_to_process function to support the new step-name and request-name parameters.
     @staticmethod
-    def assign_to_process(context: SapioWebhookContext | SapioUser, data_type: str, records: list[RecordIdentifier],
+    def assign_to_process(context: UserIdentifier, data_type: DataTypeIdentifier, records: list[RecordIdentifier],
                           process_name: str, step_number: int | None = None, branch_id: int | None = None,
-                          request: RecordIdentifier | None = None) -> None:
+                          request: RecordIdentifier | None = None, step_name: str | None = None,
+                          request_name: str | None = None) -> None:
         """
         Assign the given tracked records to a new process at the specified step, settings its status to "Ready for -"
         at that step.
         Synonymous with what occurs during request creation or when using the assign to process button.
 
+        Supports assigning to both experiment processes and custom processes, although this class is focused on
+        experiment processes. See the QueueItemHandler class for functions that support custom processes.
+
         :param context: The current webhook context or a user object to send requests from.
         :param data_type: The data type of the tracked records.
         :param records: A list of the tracked records.
@@ -23,29 +34,40 @@ class ProcessTracking:
         :param branch_id: The branch ID of the above step. Only necessary to provide if multiple steps in the process
             share the same step number.
         :param request: The request that the tracked records are a part of, to be used for the assigned process record's
-            request record ID field. If none is provided, creates a new request with default fields.
+            request record ID field. If not provided, and no request name is provided, then the tracked records will
+            not be associated with a request.
+        :param step_name: If the given process name is for a custom process, then a step name should be provided instead
+            of a step number. This is required by the system for custom processes.
+        :param request_name: The name of a new request to be created for the tracked records. Able to be provided in
+            lieu of a request ID. If not provided, and no request ID is provided, then the tracked records will
+            not be associated with a request.
         """
         sub_path = '/ext/process-tracking/assign-to-process'
         payload = {
-            "data-type-name": data_type,
+            "data-type-name": AliasUtil.to_data_type_name(data_type),
             "record-ids": AliasUtil.to_record_ids(records),
             "process-name": process_name,
             "step-number": step_number,
             "branch-id": branch_id,
-            "request-record-id": AliasUtil.to_record_ids([request])[0] if request is not None else None
+            "step-name": step_name,
+            "request-record-id": AliasUtil.to_record_id(request) if request is not None else None,
+            "request-name": request_name
         }
-        user: SapioUser = context if isinstance(context, SapioUser) else context.user
+        user: SapioUser = AliasUtil.to_sapio_user(context)
         response = user.post(sub_path, payload=payload)
         user.raise_for_status(response)
 
     @staticmethod
-    def begin_protocol(context: SapioWebhookContext | SapioUser, data_type: str, records: list[RecordIdentifier],
+    def begin_protocol(context: UserIdentifier, data_type: DataTypeIdentifier, records: list[RecordIdentifier],
                        experiment: ExperimentIdentifier) -> None:
         """
         Begin the assigned processes of the given tracked records as the given experiment. This sets the status of the
         tracked records from "Ready for -" to "In Process -" for their current step.
         Synonymous with what occurs when starting a process step in the system.
 
+        IMPORTANT NOTICE: This is only for experiment processes that make use of AssignedProcess records. For custom
+        processes that use ProcessQueueItem records, see the QueueItemHandler class.
+
         :param context: The current webhook context or a user object to send requests from.
         :param data_type: The data type of the tracked records.
         :param records: A list of the tracked records.
@@ -54,16 +76,16 @@ class ProcessTracking:
         """
         sub_path = '/ext/process-tracking/begin-protocol'
         payload = {
-            "data-type-name": data_type,
+            "data-type-name": AliasUtil.to_data_type_name(data_type),
             "record-ids": AliasUtil.to_record_ids(records),
             "experiment-id": AliasUtil.to_notebook_id(experiment),
         }
-        user: SapioUser = context if isinstance(context, SapioUser) else context.user
+        user: SapioUser = AliasUtil.to_sapio_user(context)
         response = user.post(sub_path, payload=payload)
         user.raise_for_status(response)
 
     @staticmethod
-    def complete_protocol(context: SapioWebhookContext | SapioUser, data_type: str, records: list[RecordIdentifier],
+    def complete_protocol(context: UserIdentifier, data_type: DataTypeIdentifier, records: list[RecordIdentifier],
                           experiment: ExperimentIdentifier) -> None:
         """
         Complete the current step that the given tracked records are at given the experiment.
@@ -73,6 +95,9 @@ class ProcessTracking:
         Moves the assigned process down to the descendant sample(s) if both samples are provided in the records list.
         Synonymous with what occurs when completing an experiment in the system.
 
+        IMPORTANT NOTICE: This is only for experiment processes that make use of AssignedProcess records. For custom
+        processes that use ProcessQueueItem records, see the QueueItemHandler class.
+
         :param context: The current webhook context or a user object to send requests from.
         :param data_type: The data type of the tracked records.
         :param records: A list of the tracked records.
@@ -80,22 +105,25 @@ class ProcessTracking:
         """
         sub_path = '/ext/process-tracking/complete-protocol'
         payload = {
-            "data-type-name": data_type,
+            "data-type-name": AliasUtil.to_data_type_name(data_type),
             "record-ids": AliasUtil.to_record_ids(records),
             "experiment-id": AliasUtil.to_notebook_id(experiment),
         }
-        user: SapioUser = context if isinstance(context, SapioUser) else context.user
+        user: SapioUser = AliasUtil.to_sapio_user(context)
         response = user.post(sub_path, payload=payload)
         user.raise_for_status(response)
 
     @staticmethod
-    def fail(context: SapioWebhookContext | SapioUser, data_type: str, records: list[RecordIdentifier],
+    def fail(context: UserIdentifier, data_type: DataTypeIdentifier, records: list[RecordIdentifier],
              experiment: ExperimentIdentifier) -> None:
         """
         Fail the assigned processes of the given tracked records, changing their statuses to "Failed -". The tracked
         records must be "In Process -" for the given step.
         Synonymous with what occurs when failing a sample due to a QC failure in a process in the system.
 
+        IMPORTANT NOTICE: This is only for experiment processes that make use of AssignedProcess records. For custom
+        processes that use ProcessQueueItem records, see the QueueItemHandler class.
+
         :param context: The current webhook context or a user object to send requests from.
         :param data_type: The data type of the tracked records.
         :param records: A list of the tracked records.
@@ -103,16 +131,16 @@ class ProcessTracking:
         """
         sub_path = '/ext/process-tracking/fail'
         payload = {
-            "data-type-name": data_type,
+            "data-type-name": AliasUtil.to_data_type_name(data_type),
             "record-ids": AliasUtil.to_record_ids(records),
             "experiment-id": AliasUtil.to_notebook_id(experiment),
         }
-        user: SapioUser = context if isinstance(context, SapioUser) else context.user
+        user: SapioUser = AliasUtil.to_sapio_user(context)
         response = user.post(sub_path, payload=payload)
         user.raise_for_status(response)
 
     @staticmethod
-    def promote_to_next_by_experiment(context: SapioWebhookContext | SapioUser, data_type: str,
+    def promote_to_next_by_experiment(context: UserIdentifier, data_type: DataTypeIdentifier,
                                       records: list[RecordIdentifier], experiment: ExperimentIdentifier) -> None:
         """
         Promote the status of the given tracked records to the next status in their process using an experiment.
@@ -122,6 +150,9 @@ class ProcessTracking:
         will move their status to "Ready for -" for the next step in the process, or "Completed -" if this was the
         last step.
 
+        IMPORTANT NOTICE: This is only for experiment processes that make use of AssignedProcess records. For custom
+        processes that use ProcessQueueItem records, see the QueueItemHandler class.
+
         :param context: The current webhook context or a user object to send requests from.
         :param data_type: The data type of the tracked records.
         :param records: A list of the tracked records.
@@ -129,16 +160,16 @@ class ProcessTracking:
         """
         sub_path = '/ext/process-tracking/promote-status-to-next'
         payload = {
-            "data-type-name": data_type,
+            "data-type-name": AliasUtil.to_data_type_name(data_type),
             "record-ids": AliasUtil.to_record_ids(records),
             "experiment-id": AliasUtil.to_notebook_id(experiment),
         }
-        user: SapioUser = context if isinstance(context, SapioUser) else context.user
+        user: SapioUser = AliasUtil.to_sapio_user(context)
         response = user.post(sub_path, payload=payload)
         user.raise_for_status(response)
 
     @staticmethod
-    def promote_to_next_by_step(context: SapioWebhookContext | SapioUser, data_type: str,
+    def promote_to_next_by_step(context: UserIdentifier, data_type: DataTypeIdentifier,
                                 records: list[RecordIdentifier], process_name: str, step_number: int,
                                 branch_id: int | None = None) -> None:
         """
@@ -149,6 +180,9 @@ class ProcessTracking:
         step will move their status to "Ready for -" for the next step in the process, or "Completed -" if this was the
         last step.
 
+        IMPORTANT NOTICE: This is only for experiment processes that make use of AssignedProcess records. For custom
+        processes that use ProcessQueueItem records, see the QueueItemHandler class.
+
         :param context: The current webhook context or a user object to send requests from.
         :param data_type: The data type of the tracked records.
         :param records: A list of the tracked records.
@@ -159,7 +193,7 @@ class ProcessTracking:
         """
         sub_path = '/ext/process-tracking/promote-status-to-next'
         payload = {
-            "data-type-name": data_type,
+            "data-type-name": AliasUtil.to_data_type_name(data_type),
             "record-ids": AliasUtil.to_record_ids(records),
             "current-process-status": {
                 "process-name": process_name,
@@ -167,12 +201,12 @@ class ProcessTracking:
                 "branch-id": branch_id
             }
         }
-        user: SapioUser = context if isinstance(context, SapioUser) else context.user
+        user: SapioUser = AliasUtil.to_sapio_user(context)
         response = user.post(sub_path, payload=payload)
         user.raise_for_status(response)
 
     @staticmethod
-    def reprocess(context: SapioWebhookContext | SapioUser, records: list[RecordIdentifier]) -> None:
+    def reprocess(context: UserIdentifier, records: list[RecordIdentifier]) -> None:
         """
         Reprocess tracked records to a previous step in their process. Reprocessing is controlled by ReturnPoint records
         which are children of the AssignedProcess on the tracked records. Creates a new AssignedProcess record for the
@@ -180,6 +214,9 @@ class ProcessTracking:
         Synonymous with what occurs when reprocessing records to a previous step due to QC failures in a process in the
         system.
 
+        IMPORTANT NOTICE: This is only for experiment processes that make use of AssignedProcess records. For custom
+        processes that use ProcessQueueItem records, see the QueueItemHandler class.
+
         :param context: The current webhook context or a user object to send requests from.
         :param records: A list of ReturnPoint records to reprocess to.
         """
@@ -187,6 +224,6 @@ class ProcessTracking:
         payload = {
             "record-ids": AliasUtil.to_record_ids(records)
         }
-        user: SapioUser = context if isinstance(context, SapioUser) else context.user
+        user: SapioUser = AliasUtil.to_sapio_user(context)
         response = user.post(sub_path, payload=payload)
         user.raise_for_status(response)
